@@ -1,6 +1,7 @@
 use crate::config::Config;
 use crate::converter::{CSharpTypeConverter, Converter};
 use interoptopus::lang::c::{CType, CompositeType, Constant, Documentation, EnumType, Field, FnPointerType, Function, Meta, PrimitiveType, Variant, Visibility};
+use interoptopus::patterns::callbacks::NamedCallback;
 use interoptopus::patterns::service::Service;
 use interoptopus::patterns::{LibraryPattern, TypePattern};
 use interoptopus::util::{longest_common_prefix, IdPrettifier};
@@ -80,6 +81,11 @@ pub trait CSharpWriter {
         self.write_documentation(w, function.meta().documentation())?;
         self.write_function_annotation(w, function)?;
         self.write_function_declaration(w, function)?;
+
+        w.newline()?;
+
+        self.write_function_overloaded(w, function)?;
+
         Ok(())
     }
 
@@ -114,6 +120,59 @@ pub trait CSharpWriter {
         indented!(w, r#"public static extern {} {}({});"#, rval, name, params.join(", "))
     }
 
+    #[rustfmt::skip]
+    fn write_function_overloaded(&self, w: &mut IndentWriter, function: &Function) -> Result<(), Error> {
+        // If there is nothing to write, don't do it
+        if !self.converter().has_overloadable(function.signature()) {
+            return Ok(());
+        }
+
+        let mut to_pin_name = Vec::new();
+        let mut to_pin_slice_type = Vec::new();
+        let mut to_invoke = Vec::new();
+        let rval = self.converter().function_rval_to_csharp_typename(function);
+        let name = self.converter().function_name_to_csharp_name(function);
+
+        let mut params = Vec::new();
+        for (_, p) in function.signature().params().iter().enumerate() {
+            let name = p.name();
+            let native = self.converter().pattern_to_native_in_signature(p, function.signature());
+            let the_type = self.converter().function_parameter_to_csharp_typename(p, function);
+
+            match p.the_type() {
+                CType::Pattern(TypePattern::Slice(_)) => {
+                    to_pin_name.push(name);
+                    to_pin_slice_type.push(the_type);
+                    to_invoke.push(format!("{}_slice", name));
+                }
+                _ => {
+                    to_invoke.push(name.to_string());
+                }
+            }
+
+            params.push(format!("{} {}", native, name));
+        }
+
+        indented!(w, r#"public static {} {}({}) {{"#, rval, name, params.join(", "))?;
+
+        for (pin_var, slice_struct) in to_pin_name.iter().zip(to_pin_slice_type.iter()) {
+            indented!(w, [_], r#"var {}_pinned = GCHandle.Alloc({}, GCHandleType.Pinned);"#, pin_var, pin_var)?;
+            indented!(w, [_], r#"var {}_slice = new {}({}_pinned, (ulong) {}.Length);"#, pin_var, slice_struct, pin_var, pin_var)?;
+        }
+
+        indented!(w, [_], r#"try"#)?;
+        indented!(w, [_], r#"{{"#)?;
+        indented!(w, [_ _], r#"return {}({});"#, name, to_invoke.join(", "))?;
+        indented!(w, [_], r#"}}"#)?;
+        indented!(w, [_], r#"finally"#)?;
+        indented!(w, [_], r#"{{"#)?;
+        for pin in &to_pin_name {
+            indented!(w, [_ _], r#"{}_pinned.Free();"#, pin)?;
+        }
+        indented!(w, [_], r#"}}"#)?;
+        indented!(w, r#"}}"#)
+    }
+
     fn write_type_definitions(&self, w: &mut IndentWriter) -> Result<(), Error> {
         for the_type in self.library().ctypes() {
             self.write_type_definition(w, the_type)?;
@@ -166,6 +225,15 @@ pub trait CSharpWriter {
                     if self.should_emit(x.meta()) {
                         self.write_type_definition_composite(w, x)?;
                         w.newline()?;
+                        self.write_pattern_option(w, x)?;
+                        w.newline()?;
+                    }
+                }
+                TypePattern::NamedCallback(x) => {
+                    // Handle this better way
+                    if self.should_emit(&Meta::new()) {
+                        self.write_type_definition_named_callback(w, x)?;
+                        w.newline()?;
                     }
                 }
             },
@@ -179,6 +247,24 @@ pub trait CSharpWriter {
         Ok(())
     }
 
+    fn write_type_definition_named_callback(&self, w: &mut IndentWriter, the_type: &NamedCallback) -> Result<(), Error> {
+        self.write_type_definition_fn_pointer_annotation(w, the_type.fnpointer())?;
+        self.write_type_definition_named_callback_body(w, the_type)?;
+        Ok(())
+    }
+
+    fn write_type_definition_named_callback_body(&self, w: &mut IndentWriter, the_type: &NamedCallback) -> Result<(), Error> {
+        let rval = self.converter().to_typespecifier_in_rval(the_type.fnpointer().signature().rval());
+        let name = self.converter().named_callback_to_typename(the_type);
+
+        let mut params = Vec::new();
+        for (i, param) in the_type.fnpointer().signature().params().iter().enumerate() {
+            params.push(format!("{} x{}", self.converter().to_typespecifier_in_param(param.the_type()), i));
+        }
+
+        indented!(w, r#"public delegate {} {}({});"#, rval, name, params.join(", "))
+    }
+
     fn write_type_definition_fn_pointer_annotation(&self, w: &mut IndentWriter, _the_type: &FnPointerType) -> Result<(), Error> {
         indented!(w, r#"[UnmanagedFunctionPointer(CallingConvention.Cdecl)]"#)
     }
@@ -310,6 +396,45 @@ pub trait CSharpWriter {
         Ok(())
     }
 
+    fn write_pattern_option(&self, w: &mut IndentWriter, slice: &CompositeType) -> Result<(), Error> {
+        let context_type_name = slice.rust_name();
+        let data_type = slice
+            .fields()
+            .iter()
+            .find(|x| x.name().eq("t"))
+            .expect("Option must contain field called 't'.")
+            .the_type();
+
+        let type_string = self.converter().to_typespecifier_in_rval(data_type);
+
+        indented!(w, r#"public partial struct {}"#, context_type_name)?;
+        indented!(w, r#"{{"#)?;
+
+        // FromNullable
+        indented!(w, [_], r#"public static {} FromNullable({}? nullable)"#, context_type_name, type_string)?;
+        indented!(w, [_], r#"{{"#)?;
+        indented!(w, [_ _], r#"var result = new {}();"#, context_type_name)?;
+        indented!(w, [_ _], r#"if (nullable.HasValue)"#)?;
+        indented!(w, [_ _], r#"{{"#)?;
+        indented!(w, [_ _ _], r#"result.is_some = 1;"#)?;
+        indented!(w, [_ _ _], r#"result.t = nullable.Value;"#)?;
+        indented!(w, [_ _], r#"}}"#)?;
+        w.newline()?;
+        indented!(w, [_ _], r#"return result;"#)?;
+        indented!(w, [_], r#"}}"#)?;
+        w.newline()?;
+
+        // ToNullable
+        indented!(w, [_], r#"public {}? ToNullable()"#, type_string)?;
+        indented!(w, [_], r#"{{"#)?;
+        indented!(w, [_ _], r#"return this.is_some == 1 ? this.t : ({}?)null;"#, type_string)?;
+        indented!(w, [_], r#"}}"#)?;
+
+        indented!(w, r#"}}"#)?;
+        w.newline()?;
+        Ok(())
+    }
+
     fn write_pattern_slice(&self, w: &mut IndentWriter, slice: &CompositeType) -> Result<(), Error> {
         let context_type_name = slice.rust_name();
         let data_type = slice
@@ -326,6 +451,13 @@ pub trait CSharpWriter {
         indented!(w, r#"public partial struct {} : IEnumerable<{}>"#, context_type_name, type_string)?;
         indented!(w, r#"{{"#)?;
 
+        // Ctor
+        indented!(w, [_], r#"public {}(GCHandle handle, ulong count)"#, context_type_name)?;
+        indented!(w, [_], r#"{{"#)?;
+        indented!(w, [_ _], r#"this.data = handle.AddrOfPinnedObject();"#)?;
+        indented!(w, [_ _], r#"this.len = count;"#)?;
+        indented!(w, [_], r#"}}"#)?;
+
         // Getter
         indented!(w, [_], r#"public {} this[int i]"#, type_string)?;
         indented!(w, [_], r#"{{"#)?;
